//===========================================================================
//  $Name: arts++-1-1-a4 $
//  $Id: ArtsAttributeVector.hh,v 1.2 1999/11/08 07:27:15 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSATTRIBUTEVECTOR_HH_
#define _ARTSATTRIBUTEVECTOR_HH_

#include <vector>

#include "ArtsAttribute.hh"

//----------------------------------------------------------------------------
//         class ArtsAttributeVector : public vector<ArtsAttribute> 
//----------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
class ArtsAttributeVector : public vector<ArtsAttribute>
{
public:
  //-------------------------------------------------------------------------
  //               void AddHostAttribute(ipv4addr_t hostAddr)
  //.........................................................................
  //  Adds a host attribute with an IP address of hostAddr.
  //-------------------------------------------------------------------------
  void AddHostAttribute(ipv4addr_t hostAddr);

  //-------------------------------------------------------------------------
  //    vector<ArtsAttribute>::const_iterator FindHostAttribute() const
  //.........................................................................
  //  Returns an iterator for the first host attribute in the object.
  //  If a host attribute is not found, returns this->Attributes().end()
  //-------------------------------------------------------------------------
  const_iterator FindHostAttribute() const;

  //--------------------------------------------------------------------------
  //                       bool RemoveHostAttribute() 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  bool RemoveHostAttribute();

  //--------------------------------------------------------------------------
  //  void AddHostPairAttribute(ipv4addr_t hostAddr1, ipv4addr_t hostAddr2) 
  //..........................................................................
  //  Adds a host pair attribute.
  //--------------------------------------------------------------------------
  void AddHostPairAttribute(ipv4addr_t hostAddr1, ipv4addr_t hostAddr2);

  //--------------------------------------------------------------------------
  //       const_iterator FindHostPairAttribute() const 
  //..........................................................................
  //  Returns an iterator for the first host pair attribute in the object.
  //  If a host pair attribute is not found, returns this->Attributes().end()
  //--------------------------------------------------------------------------
  const_iterator FindHostPairAttribute() const;

  //--------------------------------------------------------------------------
  //       bool RemoveHostPairAttribute() 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  bool RemoveHostPairAttribute();
  
  //-------------------------------------------------------------------------
  //            void AddCreationAttribute(uint32_t creationTime);            
  //.........................................................................
  //  Adds a creation time attribute.
  //-------------------------------------------------------------------------
  void AddCreationAttribute(uint32_t creationTime);

  //-------------------------------------------------------------------------
  //   vector<ArtsAttribute>::const_iterator FindCreationAttribute() const
  //.........................................................................
  //  Returns an iterator for the first creation attribute in the object.
  //  If a creation attribute is not found, returns this->Attributes().end()
  //-------------------------------------------------------------------------
  const_iterator FindCreationAttribute() const;

  //--------------------------------------------------------------------------
  //                     bool RemoveCreationAttribute() 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  bool RemoveCreationAttribute();
  
  //-------------------------------------------------------------------------
  //      void AddPeriodAttribute(uint32_t startTime, uint32_t endTime)
  //.........................................................................
  //  Adds a period attribute (startTime to endTime).
  //-------------------------------------------------------------------------
  void AddPeriodAttribute(uint32_t startTime, uint32_t endTime);

  //-------------------------------------------------------------------------
  //    vector<ArtsAttribute>::const_iterator FindPeriodAttribute() const
  //.........................................................................
  //  Returns an iterator for the first period attribute in the object.
  //  If a period attribute is not found, returns this->Attributes().end()
  //-------------------------------------------------------------------------
  const_iterator FindPeriodAttribute() const;

  //--------------------------------------------------------------------------
  //                      bool RemovePeriodAttribute() 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  bool RemovePeriodAttribute();
  
  //-------------------------------------------------------------------------
  //               void AddIfIndexAttribute(uint16_t ifIndex)
  //.........................................................................
  //  Adds an interface index attribute using ifIndex as value.
  //-------------------------------------------------------------------------
  void AddIfIndexAttribute(uint16_t ifIndex);

  //-------------------------------------------------------------------------
  //   vector<ArtsAttribute>::const_iterator FindIfIndexAttribute() const
  //.........................................................................
  //  returns an iterator for the first interface index attribute in
  //  the object.  If an interface index attribute is not found, returns
  //  this->Attributes().end()
  //-------------------------------------------------------------------------
  const_iterator FindIfIndexAttribute() const;

  //--------------------------------------------------------------------------
  //                      bool RemoveIfIndexAttribute() 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  bool RemoveIfIndexAttribute();
  
  //--------------------------------------------------------------------------
  //            void AddIfDescrAttribute(const string & ifDescr) 
  //..........................................................................
  //  Adds an interface description attribute using ifDescr as value.
  //--------------------------------------------------------------------------
  void AddIfDescrAttribute(const string & ifDescr);
  
  //--------------------------------------------------------------------------
  //   vector<ArtsAttribute>::const_iterator FindIfDescrAttribute() const 
  //..........................................................................
  //  returns an iterator for the first interface description attribute
  //  in the object.  If an interface description attribute is not found,
  //  returns this->Attributes().end()
  //--------------------------------------------------------------------------
  const_iterator FindIfDescrAttribute() const;

  //--------------------------------------------------------------------------
  //                      bool RemoveIfDescrAttribute() 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  bool RemoveIfDescrAttribute();
  
  //--------------------------------------------------------------------------
  //              void AddIfIpAddrAttribute(ipv4addr_t ipAddr) 
  //..........................................................................
  //  Adds an interface IP address attribute using ipAddr as value.
  //--------------------------------------------------------------------------
  void AddIfIpAddrAttribute(ipv4addr_t ipAddr);

  //--------------------------------------------------------------------------
  //   vector<ArtsAttribute>::const_iterator FindIfIpAddrAttribute() const 
  //..........................................................................
  //  returns an iterator for the first interface IP address attribute
  //  in the object.  If an interface IP address attribute is not found,
  //  returns this->Attributes().end()
  //--------------------------------------------------------------------------
  const_iterator FindIfIpAddrAttribute() const;
  
  //--------------------------------------------------------------------------
  //                     bool RemoveIfIpAddrAttribute() 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  bool RemoveIfIpAddrAttribute();
  
  //--------------------------------------------------------------------------
  //                            uint32_t Length() 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  uint32_t Length();
  
  //--------------------------------------------------------------------------
  //          istream & read(istream & is, uint16_t numAttributes) 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  istream & read(istream & is, uint16_t numAttributes);
  
  //--------------------------------------------------------------------------
  //                int read(int fd, uint16_t numAttributes) 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  int read(int fd, uint16_t numAttributes);
  
  //--------------------------------------------------------------------------
  //                      ostream & write(ostream & os) 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  ostream & write(ostream & os);
  
  //--------------------------------------------------------------------------
  //                            int write(int fd) 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  int write(int fd);
  
private:
  //--------------------------------------------------------------------------
  //               const_iterator Find(uint32_t attrType) const
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  const_iterator Find(uint32_t attrType) const;
  
  //--------------------------------------------------------------------------
  //                     bool Remove(uint32_t attrType) 
  //..........................................................................
  //  
  //--------------------------------------------------------------------------
  bool Remove(uint32_t attrType);
};

#endif  // _ARTSATTRIBUTEVECTOR_HH_
