//===========================================================================
//  @(#) $Name: arts++-1-1-a4 $
//  @(#) $Id: ArtsNetMatrixAggregator.hh,v 1.3 1999/11/09 10:24:59 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSNETMATRIXAGGREGATOR_HH_
#define _ARTSNETMATRIXAGGREGATOR_HH_

#include <map>
#include <vector>

#include "ArtsNetMatrix.hh"

class ArtsNetMatrixKeyValue
{
public:
  bool operator < (const ArtsNetMatrixKeyValue keyValue) const;
  bool operator == (const ArtsNetMatrixKeyValue keyValue) const;
  ipv4addr_t  srcNet;
  uint8_t     srcMaskLen;
  ipv4addr_t  dstNet;
  uint8_t     dstMaskLen;
};


//---------------------------------------------------------------------------
//  class ArtsNetMatrixAggregator
//---------------------------------------------------------------------------
//  This class is used to aggregate net matrix objects in the time
//  domain.
//---------------------------------------------------------------------------
class ArtsNetMatrixAggregator
{
public:
  typedef struct {
    uint64_t  Pkts;
    uint64_t  Bytes;
  } counter_t;

  //-------------------------------------------------------------------------
  //               ArtsNetMatrixAggregator(const Arts & arts)
  //.........................................................................
  //  
  //-------------------------------------------------------------------------
  ArtsNetMatrixAggregator(const Arts & arts);

  //--------------------------------------------------------------------------
  //                       ~ArtsNetMatrixAggregator()
  //..........................................................................
  //  destructor
  //--------------------------------------------------------------------------
  ~ArtsNetMatrixAggregator();

  //-------------------------------------------------------------------------
  //                       void Add(const Arts & arts)
  //.........................................................................
  //  
  //-------------------------------------------------------------------------
  void Add(const Arts & arts);
  
  //-------------------------------------------------------------------------
  //              ArtsNetMatrix *ConvertToArtsNetMatrix() const
  //.........................................................................
  //  
  //-------------------------------------------------------------------------
  ArtsNetMatrix *ConvertToArtsNetMatrix() const;
  
private:

  ArtsHeader                               _header;
  vector<ArtsAttribute>                    _attributes;
  map<ArtsNetMatrixKeyValue,counter_t,less<ArtsNetMatrixKeyValue> >  _netCounters;
  
};

#endif  // _ARTSNETMATRIXAGGREGATOR_HH_
